@@ -23,6 +23,7 @@ Copyright (C) 2010 Hiroki Ohtani(liris)
 """
 WebSocketApp provides higher level APIs.
 """
+import inspect
 import select
 import sys
 import threading
@@ -125,16 +126,17 @@ class WebSocketApp(object):
         self.url = url
         self.header = header if header is not None else []
         self.cookie = cookie
-        self.on_open = on_open
-        self.on_message = on_message
-        self.on_data = on_data
-        self.on_error = on_error
-        self.on_close = on_close
-        self.on_ping = on_ping
-        self.on_pong = on_pong
-        self.on_cont_message = on_cont_message
+
+        self.on_open = on_open or getattr(self, 'on_open', None)
+        self.on_message = on_message or getattr(self, 'on_message', None)
+        self.on_data = on_data or getattr(self, 'on_data', None)
+        self.on_error = on_error or getattr(self, 'on_error', None)
+        self.on_close = on_close or getattr(self, 'on_close', None)
+        self.on_ping = on_ping or getattr(self, 'on_ping', None)
+        self.on_pong = on_pong or getattr(self, 'on_pong', None)
+        self.on_cont_message = on_cont_message or getattr(self, 'on_cont_message', None)
+        self.get_mask_key = get_mask_key or getattr(self, 'get_mask_key', None)
         self.keep_running = False
-        self.get_mask_key = get_mask_key
         self.sock = None
         self.last_ping_tm = 0
         self.last_pong_tm = 0
@@ -159,6 +161,7 @@ class WebSocketApp(object):
         self.keep_running = False
         if self.sock:
             self.sock.close(**kwargs)
+            self.sock = None
 
     def _send_ping(self, interval, event):
         while not event.wait(interval):
@@ -175,7 +178,8 @@ class WebSocketApp(object):
                     http_proxy_host=None, http_proxy_port=None,
                     http_no_proxy=None, http_proxy_auth=None,
                     skip_utf8_validation=False,
-                    host=None, origin=None, dispatcher=None):
+                    host=None, origin=None, dispatcher=None,
+                    supress_origin = False):
         """
         run event loop for WebSocket framework.
         This loop is infinite loop and is alive during websocket is available.
@@ -193,6 +197,8 @@ class WebSocketApp(object):
         skip_utf8_validation: skip utf8 validation.
         host: update host header.
         origin: update origin header.
+        dispatcher: customize reading data from socket.
+        supress_origin: suppress outputting origin header.
         """
 
         if not ping_timeout or ping_timeout <= 0:
@@ -216,7 +222,8 @@ class WebSocketApp(object):
                 event.set()
                 thread.join()
             self.keep_running = False
-            self.sock.close()
+            if self.sock:
+                self.sock.close()
             close_args = self._get_close_args(
                 close_frame.data if close_frame else None)
             self._callback(self.on_close, *close_args)
@@ -226,14 +233,15 @@ class WebSocketApp(object):
             self.sock = WebSocket(
                 self.get_mask_key, sockopt=sockopt, sslopt=sslopt,
                 fire_cont_frame=self.on_cont_message and True or False,
-                skip_utf8_validation=skip_utf8_validation)
+                skip_utf8_validation=skip_utf8_validation,
+                enable_multithread=True if ping_interval else False)
             self.sock.settimeout(getdefaulttimeout())
             self.sock.connect(
                 self.url, header=self.header, cookie=self.cookie,
                 http_proxy_host=http_proxy_host,
                 http_proxy_port=http_proxy_port, http_no_proxy=http_no_proxy,
                 http_proxy_auth=http_proxy_auth, subprotocols=self.subprotocols,
-                host=host, origin=origin)
+                host=host, origin=origin, supress_origin = supress_origin)
             if not dispatcher:
                 dispatcher = self.create_dispatcher(ping_timeout)
 
@@ -317,7 +325,10 @@ class WebSocketApp(object):
     def _callback(self, callback, *args):
         if callback:
             try:
-                callback(self, *args)
+                if inspect.ismethod(callback):
+                    callback(*args)
+                else:
+                    callback(self, *args)
             except Exception as e:
                 _logging.error("error from callback {}: {}".format(callback, e))
                 if _logging.isEnabledForDebug():
